"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
function isObjectWalker(nodeWalker) {
    return typeof nodeWalker === "object" && nodeWalker["type"] === "object";
}
exports.isObjectWalker = isObjectWalker;
function isArrayWalker(nodeWalker) {
    return typeof nodeWalker === "object" && nodeWalker["type"] === "array";
}
exports.isArrayWalker = isArrayWalker;
function isNodeWalkerProvider(nodeWalker) {
    return typeof nodeWalker === "function";
}
exports.isNodeWalkerProvider = isNodeWalkerProvider;
exports.astWalker = {
    hookName: "ast",
    type: "object",
    children: {
        orders: {
            hookName: "orders",
            type: "array",
            child: ({ orderType }) => orderWalkers[orderType]
        }
    }
};
const orderWalkers = {
    createTable: {
        hookName: "createTable",
        type: "object",
        children: {
            entries: {
                hookName: "tableEntries",
                type: "array",
                child: ({ entryType }) => tableEntryWalkers[entryType]
            }
        }
    },
    alterTable: {
        hookName: "alterTable",
        type: "object",
        children: {
            add: {
                hookName: "tableEntries",
                type: "array",
                child: ({ entryType }) => tableEntryWalkers[entryType]
            }
        }
    },
    createIndex: {
        hookName: "createIndex",
        type: "object",
        children: {
            index: {
                hookName: "index",
                type: "object"
            }
        }
    },
};
const tableEntryWalkers = {
    column: {
        hookName: "column",
        type: "object",
        children: {
            constraints: {
                hookName: "columnConstraints",
                type: "array",
                child: ({ constraintType }) => columnConstraintWalkers[constraintType]
            }
        }
    },
    constraint: {
        hookName: "tableConstraint",
        type: "object",
        self: ({ constraintType }) => tableConstraintWalkers[constraintType]
    },
};
const tableConstraintWalkers = {
    primaryKey: {
        hookName: "primaryKeyTableConstraint",
        type: "object",
    },
    foreignKey: {
        hookName: "foreignKeyTableConstraint",
        type: "object",
    },
    unique: {
        hookName: "uniqueTableConstraint",
        type: "object",
    },
};
const columnConstraintWalkers = {
    notNull: {
        hookName: "notNullColumnConstraint",
        type: "object",
    },
    null: {
        hookName: "nullColumnConstraint",
        type: "object",
    },
    default: {
        hookName: "defaultColumnConstraint",
        type: "object",
    },
    autoincrement: {
        hookName: "autoincrementColumnConstraint",
        type: "object",
    },
    primaryKey: {
        hookName: "primaryKeyColumnConstraint",
        type: "object",
    },
    foreignKey: {
        hookName: "foreignKeyColumnConstraint",
        type: "object",
    },
    unique: {
        hookName: "uniqueColumnConstraint",
        type: "object",
    },
};
