"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const modifier_helpers_1 = require("../../ast-modifier/modifier-helpers");
exports.fkColumnToTableConstraintModifier = {
    forEach: "tableEntries",
    replace(entries) {
        const found = findAllColumnForeignKeys(entries);
        if (found.size === 0)
            return entries;
        return replaceFkColumnConstraintsWithTableConstraints(found, entries);
    }
};
function findAllColumnForeignKeys(entries) {
    const result = new Map();
    for (const entry of entries) {
        if (entry.entryType !== "column" || !entry.constraints)
            continue;
        entry.constraints.forEach((constraint, constraintIndex) => {
            if (constraint.constraintType === "foreignKey") {
                let list = result.get(entry);
                if (!list) {
                    list = [];
                    result.set(entry, list);
                }
                list.push({
                    constraintIndex,
                    fk: constraint
                });
            }
        });
    }
    return result;
}
function replaceFkColumnConstraintsWithTableConstraints(found, entries) {
    const updated = [];
    for (const entry of entries) {
        const foundFkList = entry.entryType === "column" && found.get(entry);
        if (!foundFkList)
            updated.push(entry);
        else {
            const column = entry;
            const indices = foundFkList.map(({ constraintIndex }) => constraintIndex);
            const constraints = modifier_helpers_1.removeSelectedItems(indices, column.constraints);
            const copy = Object.assign({}, column);
            if (constraints)
                copy.constraints = constraints;
            else
                delete copy.constraints;
            updated.push(copy);
        }
    }
    for (const [column, foundFkList] of found.entries()) {
        for (const { fk } of foundFkList)
            updated.push(foreignKeyColumnToTableConstraint(fk, column.name));
    }
    return updated;
}
function foreignKeyColumnToTableConstraint(fk, columnName, constraintName) {
    const tableFk = {
        entryType: "constraint",
        constraintType: "foreignKey",
        columns: [columnName],
        referencedTable: fk.referencedTable,
    };
    if (fk.referencedColumn)
        tableFk.referencedColumns = [fk.referencedColumn];
    if (fk.onDelete)
        tableFk.onDelete = fk.onDelete;
    if (fk.onUpdate)
        tableFk.onUpdate = fk.onUpdate;
    if (constraintName)
        tableFk.name = constraintName;
    return tableFk;
}
exports.foreignKeyColumnToTableConstraint = foreignKeyColumnToTableConstraint;
