"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const ast_modifier_1 = require("../ast-modifier/ast-modifier");
const gen_helpers_1 = require("./gen-helpers");
const mariadb_generator_1 = require("./mariadb-generator");
const postgresql_generator_1 = require("./postgresql-generator");
const sqlite_generator_1 = require("./sqlite-generator");
const universal_ddl_generator_1 = require("./universal-ddl-generator");
function generateDdl(ast, dialect, options = {}) {
    const maker = contextMakers[dialect.toLowerCase()];
    if (!maker)
        throw new Error(`Unknown dialect: ${dialect}`);
    const cx = maker(options);
    if (cx.createModifiers)
        ast = ast_modifier_1.modifyAst(ast, cx.createModifiers());
    let dropStatements = "";
    if (options.generateDrop && dialect !== "universalddl")
        dropStatements = makeDropStatements(ast, dialect);
    return dropStatements + gen_helpers_1.codeToString(cx, cx.gen("ast", "ast", ast));
}
exports.generateDdl = generateDdl;
const contextMakers = {
    universalddl: universal_ddl_generator_1.makeUniversalDdlGeneratorContext,
    postgresql: postgresql_generator_1.makePostgresqlDdlGeneratorContext,
    sqlite: sqlite_generator_1.makeSqliteDdlGeneratorContext,
    mariadb: mariadb_generator_1.makeMariadbDdlGeneratorContext,
};
function makeDropStatements(ast, dialect) {
    let makeDropTable;
    if (dialect === "sqlite")
        makeDropTable = (tableName) => `drop table if exists ${tableName};`;
    else
        makeDropTable = (tableName) => `drop table if exists ${tableName} cascade;`;
    const revOrders = [...ast.orders].reverse();
    const lines = [];
    for (const order of revOrders) {
        if (order.orderType === "createTable")
            lines.push(makeDropTable(order.name));
    }
    return lines.length === 0 ? "" : lines.join("\n") + "\n\n";
}
