"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const modifier_helpers_1 = require("../ast-modifier/modifier-helpers");
const column_type_1 = require("../ast-modifier/modifiers/column-type");
const gen_helpers_1 = require("./gen-helpers");
const universal_ddl_generator_1 = require("./universal-ddl-generator");
function makePostgresqlDdlGeneratorContext(options) {
    const sections = Object.assign(Object.assign({}, universal_ddl_generator_1.universalDdlSections), { tableEntries: Object.assign(Object.assign({}, universal_ddl_generator_1.universalDdlSections.tableEntries), { column(cx, node) {
                const pgc = pgColumn(node);
                if (!pgc)
                    return universal_ddl_generator_1.universalDdlSections.tableEntries.column(cx, node);
                let constraints = cx.gen("columnChildren", "constraints", pgc.constraints || []).code;
                constraints = constraints ? ` ${constraints}` : "";
                return {
                    lines: [
                        universal_ddl_generator_1.toCodeBlockComment(node.blockComment),
                        {
                            code: `${node.name} ${pgc.type}${constraints}`,
                            inlineComment: universal_ddl_generator_1.normalizeInlineComment(node.inlineComment)
                        },
                    ]
                };
            } }), columnConstraints: Object.assign(Object.assign({}, universal_ddl_generator_1.universalDdlSections.columnConstraints), { autoincrement() {
                throw new Error("Constraint 'autoincrement' must be used with 'not null' and 'integer' or 'bigint' for Postgresql");
            } }) });
    const modifier = column_type_1.makeColumnTypeReplacer(type => {
        return type === "tinyint" ? "smallint" : type;
    });
    return gen_helpers_1.makeGeneratorContext(options, sections, () => [modifier]);
}
exports.makePostgresqlDdlGeneratorContext = makePostgresqlDdlGeneratorContext;
function pgColumn(node) {
    const type = toSerialType(node.type);
    if (!type || !node.constraints)
        return;
    const found = findAutoincrementNotNullConstraints(node.constraints);
    if (!found)
        return;
    const constraints = modifier_helpers_1.removeSelectedItems(Object.values(found), node.constraints);
    return {
        type,
        constraints
    };
}
function toSerialType(type) {
    if (type === "int" || type === "integer")
        return "serial";
    if (type === "bigint")
        return "bigserial";
}
function findAutoincrementNotNullConstraints(constraints) {
    const found = constraints.reduce((found, { constraintType }, constraintIndex) => {
        if (constraintType === "autoincrement" || constraintType === "notNull")
            found[constraintType] = constraintIndex;
        return found;
    }, {});
    if (found["autoincrement"] !== undefined) {
        if (found["notNull"] === undefined)
            throw new Error("Constraint 'not null' is required with 'autoincrement' for Postgresql");
        return found;
    }
}
